package it.openutils.web.spring;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.web.servlet.handler.AbstractUrlHandlerMapping;


/**
 * Implementation of the HandlerMapping interface to map from URLs to request handler beans.
 * @author Fabrizio Giustina
 */
public class ParamNameUrlHandlerMapping extends AbstractUrlHandlerMapping
{

    /**
     * Logger.
     */
    private static Logger log = Logger.getLogger(ParamNameUrlHandlerMapping.class);

    /**
     * Map
     */
    private final Map urlMap = new HashMap();

    /**
     * Name of the param mapped to the controller.
     */
    private String paramName;

    /**
     * Set a Map with URL paths as keys and handler beans as values. Convenient for population with bean references.
     * <p>
     * Supports direct URL matches and Ant-style pattern matches. For syntax details, see the PathMatcher class.
     * @param urlMap map with URLs as keys and beans as values
     * @see org.springframework.util.PathMatcher
     */
    public void setUrlMap(Map urlMap)
    {
        this.urlMap.putAll(urlMap);
    }

    /**
     * set the parameter name
     * @param paramName
     */
    public void setParamName(String paramName)
    {
        this.paramName = paramName;
    }

    /**
     * Map URL paths to handler bean names. This the typical way of configuring this HandlerMapping.
     * <p>
     * Supports direct URL matches and Ant-style pattern matches. For syntax details, see the PathMatcher class.
     * @param mappings properties with URLs as keys and bean names as values
     * @see org.springframework.util.PathMatcher
     */
    public void setMappings(Properties mappings)
    {
        this.urlMap.putAll(mappings);
    }

    /**
     * init the application context
     * @see org.springframework.context.support.ApplicationObjectSupport#initApplicationContext()
     */
    public void initApplicationContext() throws BeansException
    {
        if (this.paramName == null)
        {
            logger.error("'paramName' not specified for ParamNameUrlHandlerMapping");
        }
        if (this.urlMap.isEmpty())
        {
            logger.info("Neither 'urlMap' nor 'mappings' set on ParamNameUrlHandlerMapping");
        }
        else
        {
            Iterator itr = this.urlMap.keySet().iterator();
            while (itr.hasNext())
            {
                String url = (String) itr.next();
                Object handler = this.urlMap.get(url);
                registerHandler(url, handler);
            }
        }
    }

    /**
     * Look up a handler for the URL path of the given request.
     * @param request current HTTP request
     * @return the looked up handler instance, or null
     */
    protected Object getHandlerInternal(HttpServletRequest request) throws Exception
    {
        String lookupPath = request.getParameter(paramName);

        if (lookupPath == null)
        {
            lookupPath = (String) request.getAttribute(paramName);
        }
        if (lookupPath == null)
        {
            log.debug("No parameter found in request for [" + paramName + "]");
            return null;
        }

        Object handler = lookupHandler(lookupPath);
        if (log.isDebugEnabled())
        {
            log.debug("Returning handler [" + handler + "] for action [" + lookupPath + "]");
        }

        return handler;
    }
}
